@@ -353,6 +353,21 @@ sysprof_capture_reader_bswap_mark (SysprofCaptureReader *self,
     mark->duration = bswap_64 (mark->duration);
 }
 
+static inline void
+sysprof_capture_reader_bswap_overlay (SysprofCaptureReader  *self,
+                                       SysprofCaptureOverlay *pr)
+{
+  assert (self != NULL);
+  assert (pr != NULL);
+
+  if (SYSPROF_UNLIKELY (self->endian != __BYTE_ORDER))
+    {
+      pr->layer = bswap_32 (pr->layer);
+      pr->src_len = bswap_32 (pr->src_len);
+      pr->dst_len = bswap_32 (pr->dst_len);
+    }
+}
+
 static inline void
 sysprof_capture_reader_bswap_jitmap (SysprofCaptureReader *self,
                                      SysprofCaptureJitmap *jitmap)
@@ -679,6 +694,54 @@ sysprof_capture_reader_read_mark (SysprofCaptureReader *self)
   return mark;
 }
 
+const SysprofCaptureOverlay *
+sysprof_capture_reader_read_overlay (SysprofCaptureReader *self)
+{
+  SysprofCaptureOverlay *pr;
+
+  assert (self != NULL);
+  assert ((self->pos % SYSPROF_CAPTURE_ALIGN) == 0);
+  assert (self->pos <= self->bufsz);
+
+  if (!sysprof_capture_reader_ensure_space_for (self, sizeof *pr + 1))
+    return NULL;
+
+  pr = (SysprofCaptureOverlay *)(void *)&self->buf[self->pos];
+
+  sysprof_capture_reader_bswap_frame (self, &pr->frame);
+
+  if (pr->frame.type != SYSPROF_CAPTURE_FRAME_OVERLAY)
+    return NULL;
+
+  if (pr->frame.len < (sizeof *pr + 2))
+    return NULL;
+
+  if (!sysprof_capture_reader_ensure_space_for (self, pr->frame.len))
+    return NULL;
+
+  pr = (SysprofCaptureOverlay *)(void *)&self->buf[self->pos];
+
+  sysprof_capture_reader_bswap_overlay (self, pr);
+
+  /* Make sure there is enough space for paths and trailing \0 */
+  if (((size_t)pr->src_len + (size_t)pr->dst_len) > (pr->frame.len - sizeof *pr - 2))
+    return NULL;
+
+  /* Enforce trailing \0 */
+  pr->data[pr->src_len] = 0;
+  pr->data[pr->src_len+1+pr->dst_len] = 0;
+
+  self->pos += pr->frame.len;
+
+  if ((self->pos % SYSPROF_CAPTURE_ALIGN) != 0)
+    return NULL;
+
+  /* Ensure trailing \0 in name and message */
+  ((char *)pr)[pr->frame.len-1] = 0;
+
+  return pr;
+}
+
 const SysprofCaptureMetadata *
 sysprof_capture_reader_read_metadata (SysprofCaptureReader *self)
 {
